/* WARNING: Do NOT edit this file.  It was created automatically
   with the command "make bibclean.h" by beebe@plot79.math.utah.edu
   in /u/sy/beebe/tex/bibclean/2-11 on Fri May 3 09:59:30 MDT 1996 */

	"Command-line switches may be abbreviated to a unique leading\n",
	"prefix, and letter case is not significant.  All options are parsed\n",
	"before any input bibliography files are read, no matter what their\n",
	"order on the command line.  Options that correspond to a yes/no\n",
	"setting of a flag have a form with a prefix \"no-\" to set the flag\n",
	"to no.  For such options, the last setting determines the flag\n",
	"value used.  This is significant when options are also specified in\n",
	"initialization files (see the INITIALIZATION FILES manual section).\n",
	"\n",
	"-author    Display an author credit on the standard error unit,\n",
	"          stderr.  Sometimes an executable program is separated\n",
	"          from its documentation and source code; this option\n",
	"          provides a way to recover from that.\n",
	"\n",
	"-error-log filename\n",
	"          Redirect stderr to the indicated file, which will then\n",
	"          contain all of the error and warning messages.  This\n",
	"          option is provided for those systems that have difficulty\n",
	"          redirecting stderr.\n",
	"\n",
	"-help or -?\n",
	"          Display a help message on stderr, giving a usage\n",
	"          description, similar to this section of the manual pages.\n",
	"\n",
	"-init-file filename\n",
	"          Provide an explicit value pattern initialization file.\n",
	"          It will be processed after any system-wide and job-wide\n",
	"          initialization files found on the PATH (for VAX VMS,\n",
	"          SYS$SYSTEM) and BIBINPUTS search paths, respectively, and\n",
	"          may override them.  It in turn may be overridden by a\n",
	"          subsequent file-specific initialization file.  The\n",
	"          initialization file name can be changed at compile time,\n",
	"          or at run time through a setting of the environment\n",
	"          variable BIBCLEANINI, but defaults to .bibcleanrc on\n",
	"          UNIX, and to bibclean.ini elsewhere.  For further\n",
	"          details, see the INITIALIZATION FILES manual section.\n",
	"\n",
	"-max-width nnn\n",
	"          bibclean normally limits output line widths to 72\n",
	"          characters, and in the interests of consistency, that\n",
	"          value should not be changed.  Occasionally, special-\n",
	"          purpose applications may require different maximum line\n",
	"          widths, so this option provides that capability.  The\n",
	"          number following the option name can be specified in\n",
	"          decimal, octal (starting with 0), or hexadecimal\n",
	"          (starting with 0x).  A zero or negative value is\n",
	"          interpreted to mean unlimited, so -max-width 0 can be\n",
	"          used to ensure that each field/value pair appears on a\n",
	"          single line.\n",
	"\n",
	"          When -no-prettyprint requests bibclean to act as a\n",
	"          lexical analyzer, the default line width is unlimited,\n",
	"          unless overridden by this option.\n",
	"\n",
	"          When bibclean is prettyprinting, line wrapping will be\n",
	"          done only at a space. Consequently, a long non-blank\n",
	"          character sequence may result in the output exceeding the\n",
	"          requested line width.\n",
	"\n",
	"          When bibclean is lexing, line wrapping is done by\n",
	"          inserting a backslash-newline pair when the specified\n",
	"          maximum is reached, so no line length will ever exceed\n",
	"          the maximum.\n",
	"\n",
	"-[no-]align-equals\n",
	"          With the positive form, align the equals sign in\n",
	"          key/value assignments at the same column, separated by a\n",
	"          single space from the value string.  Otherwise, the\n",
	"          equals sign follows the key, separated by a single space.\n",
	"          Default: no.\n",
	"\n",
	"-[no-]check-values\n",
	"          With the positive form, apply heuristic pattern matching\n",
	"          to field values in order to detect possible errors (e.g.\n",
	"          ``year = \"192\"'' instead of ``year = \"1992\"''), and issue\n",
	"          warnings when unexpected patterns are found.\n",
	"\n",
	"          This checking is usually beneficial, but if it produces\n",
	"          too many bogus warnings for a particular bibliography\n",
	"          file, you can disable it with the negative form of this\n",
	"          option.  Default: yes.\n",
	"\n",
	"-[no-]delete-empty-values\n",
	"          With the positive form, remove all field/value pairs for\n",
	"          which the value is an empty string.  This is helpful in\n",
	"          cleaning up bibliographies generated from text editor\n",
	"          templates. Compare this option with -[no-]remove-OPT-\n",
	"          prefixes described below.  Default: no.\n",
	"\n",
	"-[no-]file-position\n",
	"          With the positive form, give detailed file position\n",
	"          information in warning and error messages.  Default: no.\n",
	"\n",
	"-[no-]fix-font-changes\n",
	"          With the positive form, supply an additional brace level\n",
	"          around font changes in titles to protect against\n",
	"          downcasing by some BibTeX styles.  Font changes that\n",
	"          already have more than one level of braces are not\n",
	"          modified.\n",
	"\n",
	"          For example, if a title contains the Latin phrase {\\em\n",
	"          Dictyostelium Discoideum} or {\\em {D}ictyostelium\n",
	"          {D}iscoideum}, then downcasing will incorrectly convert\n",
	"          the phrase to lower-case letters.  Most BibTeX users are\n",
	"          surprised that bracing the initial letters does not\n",
	"          prevent the downcase action.  The correct coding is {{\\em\n",
	"          Dictyostelium Discoideum}}.  However, there are also\n",
	"          legitimate cases where an extra level of bracing wrongly\n",
	"          protects from downcasing.  Consequently, bibclean will\n",
	"          normally not supply an extra level of braces, but if you\n",
	"          have a bibliography where the extra braces are routinely\n",
	"          missing, you can use this option to supply them.\n",
	"\n",
	"          If you think that you need this option, it is strongly\n",
	"          recommended that you apply bibclean to your bibliography\n",
	"          file with and without -fix-font-changes, then compare the\n",
	"          two output files to ensure that extra braces are not\n",
	"          being supplied in titles where they should not be\n",
	"          present.  You will have to decide which of the two output\n",
	"          files is the better choice, then repair the incorrect\n",
	"          title bracing by hand.\n",
	"\n",
	"          Since font changes in titles are uncommon, except for\n",
	"          cases of the type which this option is designed to\n",
	"          correct, it should do more good than harm.  Default: no.\n",
	"\n",
	"-[no-]fix-initials\n",
	"          With the positive form, insert a space after a period\n",
	"          following author initials.  Default: yes.\n",
	"\n",
	"-[no-]fix-names\n",
	"          With the positive form, reorder author and editor name\n",
	"          lists to remove commas at brace level zero, placing first\n",
	"          names or initials before last names.  Default: yes.\n",
	"\n",
	"-[no-]German-style\n",
	"          With the positive form, interpret quote characters [\"]\n",
	"          inside braced value strings at brace level 1 according to\n",
	"          the conventions of the TeX style file german.sty, which\n",
	"          overloads quote to simplify input and representation of\n",
	"          German umlaut accents, sharp-s (es-zet), ligature\n",
	"          separators, invisible hyphens, raised/lowered quotes,\n",
	"          French guillemets, and discretionary hyphens.  Recognized\n",
	"          character combinations will be braced to prevent BibTeX\n",
	"          from interpreting the quote as a string delimiter.\n",
	"\n",
	"          Quoted strings receive no special handling from this\n",
	"          option, and since German nouns in titles must anyway be\n",
	"          protected from the downcasing operation of most BibTeX\n",
	"          bibliography styles, German value strings that use the\n",
	"          overloaded quote character can always be entered in the\n",
	"          form \"{...}\", without the need to specify this option at\n",
	"          all.\n",
	"\n",
	"          Default: no.\n",
	"\n",
	"-[no-]keep-linebreaks\n",
	"          Normally, line breaks inside value strings are collapsed\n",
	"          into a single space, so that long value strings can later\n",
	"          be broken to provide lines of reasonable length.\n",
	"\n",
	"          With the positive form, linebreaks are preserved in value\n",
	"          strings.  If -max-width is set to zero, this preserves\n",
	"          the original line breaks.  Spacing outside value strings\n",
	"          remains under bibclean's control, and is not affected by\n",
	"          this option.\n",
	"\n",
	"          Default: no.\n",
	"\n",
	"-[no-]keep-parbreaks\n",
	"          With the positive form, preserve paragraph breaks (either\n",
	"          formfeeds, or lines containing only spaces) in value\n",
	"          strings.  Normally, paragraph breaks are collapsed into a\n",
	"          single space.  Spacing outside value strings remains\n",
	"          under bibclean's control, and is not affected by this\n",
	"          option.  Default: no.\n",
	"\n",
	"-[no-]keep-preamble-spaces\n",
	"          With the positive form, preserve all whitespace in\n",
	"          @Preamble{...} entries.  Default: no.\n",
	"\n",
	"-[no-]keep-spaces\n",
	"          With the positive form, preserve all spaces in value\n",
	"          strings.  Normally, multiple spaces are collapsed into a\n",
	"          single space.  This option can be used together with -\n",
	"          keep-linebreaks, -keep-parbreaks, and -max-width 0 to\n",
	"          preserve the form of value strings while still providing\n",
	"          syntax and value checking.  Spacing outside value strings\n",
	"          remains under bibclean's control, and is not affected by\n",
	"          this option.  Default: no.\n",
	"\n",
	"-[no-]keep-string-spaces\n",
	"          With the positive form, preserve all whitespace in\n",
	"          @String{...} entries.  Default: no.\n",
	"\n",
	"-[no-]parbreaks\n",
	"          With the negative form, a paragraph break (either a\n",
	"          formfeed, or a line containing only spaces) is not\n",
	"          permitted in value strings, or between field/value pairs.\n",
	"          This may be useful to quickly trap runaway strings\n",
	"          arising from mismatched delimiters.  Default: yes.\n",
	"\n",
	"-[no-]prettyprint\n",
	"          Normally, bibclean functions as a prettyprinter.\n",
	"          However, with the negative form of this option, it acts\n",
	"          as a lexical analyzer instead, producing a stream of\n",
	"          lexical tokens.  See the LEXICAL ANALYSIS manual section\n",
	"          for further details.  Default: yes.\n",
	"\n",
	"-[no-]print-patterns\n",
	"          With the positive form, print the value patterns read\n",
	"          from initialization files as they are added to internal\n",
	"          tables.  Use this option to check newly-added patterns,\n",
	"          or to see what patterns are being used.\n",
	"\n",
	"          When bibclean is compiled with native pattern-matching\n",
	"          code (the default), these patterns are the ones that will\n",
	"          be used in checking value strings for valid syntax, and\n",
	"          all of them are specified in initialization files, rather\n",
	"          than hard-coded into the program.  For further details,\n",
	"          see the INITIALIZATION FILES manual section.  Default:\n",
	"          no.\n",
	"\n",
	"-[no-]read-init-files\n",
	"          With the negative form, suppress loading of system-,\n",
	"          user-, and file-specific initialization files.\n",
	"          Initializations will come only from those files\n",
	"          explicitly given by -init-file filename options.\n",
	"          Default: yes.\n",
	"\n",
	"-[no-]remove-OPT-prefixes\n",
	"          With the positive form, remove the ``OPT'' prefix from\n",
	"          each field name where the corresponding value is not an\n",
	"          empty string.  The prefix ``OPT'' must be entirely in\n",
	"          upper-case to be recognized.\n",
	"\n",
	"          This option is for bibliographies generated with the help\n",
	"          of the GNU Emacs BibTeX editing support, which generates\n",
	"          templates with optional fields identified by the ``OPT''\n",
	"          prefix.  Although the function M-x bibtex-remove-OPT\n",
	"          normally bound to the keystrokes C-c C-o does the job,\n",
	"          users often forget, with the result that BibTeX does not\n",
	"          recognize the field name, and ignores the value string.\n",
	"          Compare this option with -[no-]delete-empty-values\n",
	"          described above.  Default: no.\n",
	"\n",
	"-[no-]scribe\n",
	"          With the positive form, accept input syntax conforming to\n",
	"          the Scribe document system.  The output will be converted\n",
	"          to conform to BibTeX syntax.  See the SCRIBE BIBLIOGRAPHY\n",
	"          FORMAT manual section for further details.  Default: no.\n",
	"\n",
	"-[no-]trace-file-opening\n",
	"          With the positive form, record in the error log file the\n",
	"          names of all files which bibclean attempts to open.  Use\n",
	"          this option to identify where initialization files are\n",
	"          located.  Default: no.\n",
	"\n",
	"-[no-]warnings\n",
	"          With the positive form, allow all warning messages.  The\n",
	"          negative form is not recommended since it may mask\n",
	"          problems that should be repaired.  Default: yes.\n",
	"\n",
	"-version   Display the program version number on stderr.  This will\n",
	"          also include an indication of who compiled the program,\n",
	"          the host name on which it was compiled, the time of\n",
	"          compilation, and the type of string-value matching code\n",
	"          selected, when that information is available to the\n",
	"          compiler.\n",
	"\n",
	(const char*)NULL,
