#ifndef TOKEN_H_DEFINED_
#define TOKEN_H_DEFINED_

#if !defined(MAX_TOKEN)
#define MAX_TOKEN	4093	/* internal buffer size; no BibTeX string
				value may be larger than this. */
#endif /* !defined(MAX_TOKEN) */

#define MAX_TOKEN_SIZE	(MAX_TOKEN + 3)	/* Arrays are always dimensioned
				MAX_TOKEN_SIZE, so as to have space
				for an additional pair of braces and a
				trailing NUL, without tedious
				subscript checking in inner loops. */

#endif /* TOKEN_H_DEFINED_ */
