#ifndef XUNISTD_H_DEFINED_
#define XUNISTD_H_DEFINED_

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_LIBC_H
#include <libc.h>			/* e.g. NeXT Mach 3.x system */
#endif

#endif /* XUNISTD_H_DEFINED_ */
